require "actions/context"
require "actions/safe_context"
require "actions/action/error_handling"
require "actions/action/storage"
require "json"

module Actions
  module Action

    def self.included(base)
      base.class_eval do
        prepend Storage
        prepend ErrorHandling
        extend ClassMethods
        attr_reader :context
      end
    end

    module ClassMethods

      def call(context={})
        new(context).call
      end

      def input(attribute, type: Object, required: false, mutable: false)
        self.inputs[attribute.to_s] = { type: type, mutable: mutable, required: required }
      end

      def output(attribute, type: Object)
        self.outputs[attribute.to_s] = { type: type }
      end

      def inputs
        @inputs ||= {}
      end

      def outputs
        @outputs ||= {}
      end

    end

    # FIXME: Handle context failures more gracefully, i.e. don't initialize a new
    # action if one in the group failed previously.
    def initialize(context = {})
      @context = SafeContext.new(context)
      if @context.success? && (self.class.inputs.any? || self.class.outputs.any?)
        @context.action = self
        self.class.inputs.each do |name, options|
          @context.input!(name, type: options[:type], mutable: options[:mutable], required: options[:required])
        end
        self.class.outputs.each do |name, options|
          @context.output!(name, type: options[:type])
        end
      else
        @context = Context.build(context)
        @context.action = self
      end
    end

  end
end
